@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -59,8 +59,7 @@ static inline Quantum GetPixelAlpha(const Image *magick_restrict image,
   return(pixel[image->channel_map[AlphaPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelAlphaTraits(
-  const Image *magick_restrict image)
+static inline PixelTrait GetPixelAlphaTraits(const Image *magick_restrict image)
 {
   return(image->channel_map[AlphaPixelChannel].traits);
 }
@@ -79,8 +78,7 @@ static inline Quantum GetPixelBlack(const Image *magick_restrict image,
   return(pixel[image->channel_map[BlackPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelBlackTraits(
-  const Image *magick_restrict image)
+static inline PixelTrait GetPixelBlackTraits(const Image *magick_restrict image)
 {
   return(image->channel_map[BlackPixelChannel].traits);
 }
@@ -138,14 +136,20 @@ static inline size_t GetPixelChannels(const Image *magick_restrict image)
   return(image->number_channels);
 }
 
-static inline Quantum GetPixelCompositeMask(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline Quantum GetPixelCompositeMask(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[CompositeMaskPixelChannel].traits == UndefinedPixelTrait)
     return((Quantum) QuantumRange);
   return(pixel[image->channel_map[CompositeMaskPixelChannel].offset]);
 }
 
+static inline PixelTrait GetPixelCompositeMaskTraits(
+  const Image *magick_restrict image)
+{
+  return(image->channel_map[CompositeMaskPixelChannel].traits);
+}
+
 static inline Quantum GetPixelCr(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
@@ -185,8 +189,7 @@ static inline Quantum GetPixelGreen(const Image *magick_restrict image,
   return(pixel[image->channel_map[GreenPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelGreenTraits(
-  const Image *magick_restrict image)
+static inline PixelTrait GetPixelGreenTraits(const Image *magick_restrict image)
 {
   return(image->channel_map[GreenPixelChannel].traits);
 }
@@ -199,8 +202,7 @@ static inline Quantum GetPixelIndex(const Image *magick_restrict image,
   return(pixel[image->channel_map[IndexPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelIndexTraits(
-  const Image *magick_restrict image)
+static inline PixelTrait GetPixelIndexTraits(const Image *magick_restrict image)
 {
   return(image->channel_map[IndexPixelChannel].traits);
 }
@@ -292,8 +294,8 @@ static inline ssize_t GetPixelLabel(const Image *magick_restrict image,
   return((ssize_t) pixel[image->channel_map[LabelPixelChannel].offset]);
 }
 
-static inline MagickRealType GetPixelLuma(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline MagickRealType GetPixelLuma(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   MagickRealType
     intensity;
@@ -340,6 +342,19 @@ static inline PixelTrait GetPixelMagentaTraits(
   return(image->channel_map[MagentaPixelChannel].traits);
 }
 
+static inline Quantum GetPixelMeta(const Image *magick_restrict image,
+  const Quantum *magick_restrict pixel)
+{
+  if (image->channel_map[MetaPixelChannel].traits == UndefinedPixelTrait)
+    return(OpaqueAlpha);
+  return(pixel[image->channel_map[MetaPixelChannel].offset]);
+}
+
+static inline PixelTrait GetPixelMetaTraits(const Image *magick_restrict image)
+{
+  return(image->channel_map[MetaPixelChannel].traits);
+}
+
 static inline Quantum GetPixelReadMask(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
@@ -348,8 +363,8 @@ static inline Quantum GetPixelReadMask(const Image *magick_restrict image,
   return(pixel[image->channel_map[ReadMaskPixelChannel].offset]);
 }
 
-static inline Quantum GetPixelWriteMask(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline Quantum GetPixelWriteMask(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[WriteMaskPixelChannel].traits == UndefinedPixelTrait)
     return((Quantum) QuantumRange);
@@ -435,6 +450,12 @@ static inline PixelTrait GetPixelTraits(const Image *magick_restrict image,
   return(image->channel_map[channel].traits);
 }
 
+static inline PixelTrait GetPixelWriteMaskTraits(
+  const Image *magick_restrict image)
+{
+  return(image->channel_map[WriteMaskPixelChannel].traits);
+}
+
 static inline Quantum GetPixelY(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
@@ -476,7 +497,7 @@ static inline MagickBooleanType IsPixelAtDepth(const Quantum pixel,
     (((MagickRealType) range*pixel)/QuantumRange+0.5)))/range+0.5);
 #else
   quantum=(Quantum) (((MagickRealType) QuantumRange*((QuantumAny)
-    (((MagickRealType) range*pixel)/QuantumRange+0.5)))/range);
+    (((MagickRealType) range*pixel)/QuantumRange+0.5)))/(MagickRealType) range);
 #endif
   return(pixel == quantum ? MagickTrue : MagickFalse);
 }
@@ -832,6 +853,17 @@ static inline void SetPixelMagentaTraits(Image *image,const PixelTrait traits)
   image->channel_map[MagentaPixelChannel].traits=traits;
 }
 
+static inline void SetPixelMeta(const Image *magick_restrict image,
+  const Quantum red,Quantum *magick_restrict pixel)
+{
+  pixel[image->channel_map[MetaPixelChannel].offset]=red;
+}
+
+static inline void SetPixelMetaTraits(Image *image,const PixelTrait traits)
+{
+  image->channel_map[MetaPixelChannel].traits=traits;
+}
+
 static inline void SetPixelReadMask(const Image *magick_restrict image,
   const Quantum mask,Quantum *magick_restrict pixel)
 {
