@@ -16,7 +16,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "MagickCore/configure-private.h"
 #include "MagickCore/exception.h"
 #include "MagickCore/exception-private.h"
+#include "MagickCore/linked-list-private.h"
 #include "MagickCore/magick-private.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/memory-private.h"
@@ -176,7 +177,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
   MagickStatusType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -184,7 +185,8 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
   */
   cache=NewLinkedList(0);
   status=MagickTrue;
-#if defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
+  (void) filename;
   status=LoadPolicyCache(cache,ZeroConfigurationPolicy,"[zero-configuration]",0,
     exception);
 #else
@@ -214,7 +216,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
     PolicyInfo
       *policy_info;
 
-    register const PolicyMapInfo
+    const PolicyMapInfo
       *p;
 
     p=PolicyMap+i;
@@ -276,10 +278,13 @@ static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
   PolicyDomain
     domain;
 
-  register PolicyInfo
+  PolicyInfo
+    *policy;
+
+  ElementInfo
     *p;
 
-  register char
+  char
     *q;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -315,26 +320,31 @@ static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
   /*
     Search for policy tag.
   */
+  policy=(PolicyInfo *) NULL;
   LockSemaphoreInfo(policy_semaphore);
   ResetLinkedListIterator(policy_cache);
-  p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
+  p=GetHeadElementInLinkedList(policy_cache);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
     {
       UnlockSemaphoreInfo(policy_semaphore);
-      return(p);
+      if (p != (ElementInfo *) NULL)
+        policy=(PolicyInfo *) p->value;
+      return(policy);
     }
-  while (p != (PolicyInfo *) NULL)
+  while (p != (ElementInfo *) NULL)
   {
-    if ((domain == UndefinedPolicyDomain) || (p->domain == domain))
-      if (LocaleCompare(policyname,p->name) == 0)
+    policy=(PolicyInfo *) p->value;
+    if ((domain == UndefinedPolicyDomain) || (policy->domain == domain))
+      if (LocaleCompare(policyname,policy->name) == 0)
         break;
-    p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
+    p=p->next;
   }
-  if (p != (PolicyInfo *) NULL)
-    (void) InsertValueInLinkedList(policy_cache,0,
-      RemoveElementByValueFromLinkedList(policy_cache,p));
+  if (p == (ElementInfo *) NULL)
+    policy=(PolicyInfo *) NULL;
+  else
+    (void) SetHeadElementInLinkedList(policy_cache,p);
   UnlockSemaphoreInfo(policy_semaphore);
-  return(p);
+  return(policy);
 }
 
 /*
@@ -370,41 +380,41 @@ MagickExport const PolicyInfo **GetPolicyInfoList(const char *pattern,
   const PolicyInfo
     **policies;
 
-  register const PolicyInfo
+  ElementInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  /*
-    Allocate policy list.
-  */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_policies != (size_t *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   *number_policies=0;
-  p=GetPolicyInfo("*",exception);
-  if (p == (const PolicyInfo *) NULL)
+  if (IsPolicyCacheInstantiated(exception) == MagickFalse)
     return((const PolicyInfo **) NULL);
   policies=(const PolicyInfo **) AcquireQuantumMemory((size_t)
     GetNumberOfElementsInLinkedList(policy_cache)+1UL,sizeof(*policies));
   if (policies == (const PolicyInfo **) NULL)
     return((const PolicyInfo **) NULL);
-  /*
-    Generate policy list.
-  */
   LockSemaphoreInfo(policy_semaphore);
-  ResetLinkedListIterator(policy_cache);
-  p=(const PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  for (i=0; p != (const PolicyInfo *) NULL; )
+  p=GetHeadElementInLinkedList(policy_cache);
+  for (i=0; p != (ElementInfo *) NULL; )
   {
-    if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
-      policies[i++]=p;
-    p=(const PolicyInfo *) GetNextValueInLinkedList(policy_cache);
+    const PolicyInfo
+      *policy;
+
+    policy=(const PolicyInfo *)p->value;
+    if ((policy->stealth == MagickFalse) &&
+        (GlobExpression(policy->name,pattern,MagickFalse) != MagickFalse))
+      policies[i++]=policy;
+    p=p->next;
   }
   UnlockSemaphoreInfo(policy_semaphore);
-  policies[i]=(PolicyInfo *) NULL;
+  if (i == 0)
+    policies=(const PolicyInfo **) RelinquishMagickMemory(policies);
+  else
+    policies[i]=(PolicyInfo *) NULL;
   *number_policies=(size_t) i;
   return(policies);
 }
@@ -436,47 +446,71 @@ MagickExport const PolicyInfo **GetPolicyInfoList(const char *pattern,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport char **GetPolicyList(const char *pattern,
-  size_t *number_policies,ExceptionInfo *exception)
+
+static char *AcquirePolicyString(const char *source,const size_t pad)
+{
+  char
+    *destination;
+
+  size_t
+    length;
+
+  length=0;
+  if (source != (char *) NULL)
+    length+=strlen(source);
+  destination=(char *) NULL;
+  /* AcquireMagickMemory needs to be used here to avoid an omp deadlock */
+  if (~length >= pad)
+    destination=(char *) AcquireMagickMemory((length+pad)*sizeof(*destination));
+  if (destination == (char *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
+  if (source != (char *) NULL)
+    (void) memcpy(destination,source,length*sizeof(*destination));
+  destination[length]='\0';
+  return(destination);
+}
+
+MagickExport char **GetPolicyList(const char *pattern,size_t *number_policies,
+  ExceptionInfo *exception)
 {
   char
     **policies;
 
-  register const PolicyInfo
+  const ElementInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  /*
-    Allocate policy list.
-  */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_policies != (size_t *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   *number_policies=0;
-  p=GetPolicyInfo("*",exception);
-  if (p == (const PolicyInfo *) NULL)
+  if (IsPolicyCacheInstantiated(exception) == MagickFalse)
     return((char **) NULL);
   policies=(char **) AcquireQuantumMemory((size_t)
     GetNumberOfElementsInLinkedList(policy_cache)+1UL,sizeof(*policies));
   if (policies == (char **) NULL)
     return((char **) NULL);
-  /*
-    Generate policy list.
-  */
   LockSemaphoreInfo(policy_semaphore);
-  ResetLinkedListIterator(policy_cache);
-  p=(const PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  for (i=0; p != (const PolicyInfo *) NULL; )
+  p=GetHeadElementInLinkedList(policy_cache);
+  for (i=0; p != (ElementInfo *) NULL; )
   {
-    if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
-      policies[i++]=ConstantString(p->name);
-    p=(const PolicyInfo *) GetNextValueInLinkedList(policy_cache);
+    const PolicyInfo
+      *policy;
+
+    policy=(const PolicyInfo *) p->value;
+    if ((policy->stealth == MagickFalse) &&
+        (GlobExpression(policy->name,pattern,MagickFalse) != MagickFalse))
+      policies[i++]=AcquirePolicyString(policy->name,1);
+    p=p->next;
   }
   UnlockSemaphoreInfo(policy_semaphore);
-  policies[i]=(char *) NULL;
+  if (i == 0)
+    policies=(char **) RelinquishMagickMemory(policies);
+  else
+    policies[i]=(char *) NULL;
   *number_policies=(size_t) i;
   return(policies);
 }
@@ -515,7 +549,8 @@ MagickExport char *GetPolicyValue(const char *name)
     *exception;
 
   assert(name != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",name);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",name);
   exception=AcquireExceptionInfo();
   policy_info=GetPolicyInfo(name,exception);
   exception=DestroyExceptionInfo(exception);
@@ -524,7 +559,7 @@ MagickExport char *GetPolicyValue(const char *name)
   value=policy_info->value;
   if ((value == (const char *) NULL) || (*value == '\0'))
     return((char *) NULL);
-  return(ConstantString(value));
+  return(AcquirePolicyString(value,1));
 }
 
 /*
@@ -554,6 +589,7 @@ static MagickBooleanType IsPolicyCacheInstantiated(ExceptionInfo *exception)
 {
   if (policy_cache == (LinkedListInfo *) NULL)
     {
+      GetMaxMemoryRequest();  /* avoid OMP deadlock */
       if (policy_semaphore == (SemaphoreInfo *) NULL)
         ActivateSemaphoreInfo(&policy_semaphore);
       LockSemaphoreInfo(policy_semaphore);
@@ -604,10 +640,10 @@ MagickExport MagickBooleanType IsRightsAuthorized(const PolicyDomain domain,
   MagickBooleanType
     authorized;
 
-  register PolicyInfo
+  ElementInfo
     *p;
 
-  if (IsEventLogging() != MagickFalse)
+  if ((GetLogEventMask() & PolicyEvent) != 0)
     (void) LogMagickEvent(PolicyEvent,GetMagickModule(),
       "Domain: %s; rights=%s; pattern=\"%s\" ...",
       CommandOptionToMnemonic(MagickPolicyDomainOptions,domain),
@@ -619,24 +655,27 @@ MagickExport MagickBooleanType IsRightsAuthorized(const PolicyDomain domain,
     return(MagickTrue);
   authorized=MagickTrue;
   LockSemaphoreInfo(policy_semaphore);
-  ResetLinkedListIterator(policy_cache);
-  p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  while (p != (PolicyInfo *) NULL)
+  p=GetHeadElementInLinkedList(policy_cache);
+  while (p != (ElementInfo *) NULL)
   {
-    if ((p->domain == domain) &&
-        (GlobExpression(pattern,p->pattern,MagickFalse) != MagickFalse))
+    const PolicyInfo
+      *policy;
+
+    policy=(const PolicyInfo *) p->value;
+    if ((policy->domain == domain) &&
+        (GlobExpression(pattern,policy->pattern,MagickFalse) != MagickFalse))
       {
         if ((rights & ReadPolicyRights) != 0)
-          authorized=(p->rights & ReadPolicyRights) != 0 ? MagickTrue :
+          authorized=(policy->rights & ReadPolicyRights) != 0 ? MagickTrue :
             MagickFalse;
         if ((rights & WritePolicyRights) != 0)
-          authorized=(p->rights & WritePolicyRights) != 0 ? MagickTrue :
+          authorized=(policy->rights & WritePolicyRights) != 0 ? MagickTrue :
             MagickFalse;
         if ((rights & ExecutePolicyRights) != 0)
-          authorized=(p->rights & ExecutePolicyRights) != 0 ? MagickTrue :
+          authorized=(policy->rights & ExecutePolicyRights) != 0 ? MagickTrue :
             MagickFalse;
       }
-    p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
+    p=p->next;
   }
   UnlockSemaphoreInfo(policy_semaphore);
   return(authorized);
@@ -676,7 +715,7 @@ MagickExport MagickBooleanType ListPolicyInfo(FILE *file,
   const PolicyInfo
     **policy_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -793,7 +832,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
     return(MagickFalse);
   status=MagickTrue;
   policy_info=(PolicyInfo *) NULL;
-  token=AcquireString(xml);
+  token=AcquirePolicyString(xml,MagickPathExtent);
   extent=strlen(token)+MagickPathExtent;
   for (q=(const char *) xml; *q != '\0'; )
   {
@@ -856,8 +895,8 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
                   file_xml=FileToXML(path,~0UL);
                   if (file_xml != (char *) NULL)
                     {
-                      status&=LoadPolicyCache(cache,file_xml,path,
-                        depth+1,exception);
+                      status&=LoadPolicyCache(cache,file_xml,path,depth+1,
+                        exception);
                       file_xml=DestroyString(file_xml);
                     }
                 }
@@ -872,7 +911,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
         */
         policy_info=(PolicyInfo *) AcquireCriticalMemory(sizeof(*policy_info));
         (void) memset(policy_info,0,sizeof(*policy_info));
-        policy_info->path=ConstantString(filename);
+        policy_info->path=AcquirePolicyString(filename,1);
         policy_info->exempt=MagickFalse;
         policy_info->signature=MagickCoreSignature;
         continue;
@@ -913,7 +952,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            policy_info->name=ConstantString(token);
+            policy_info->name=AcquirePolicyString(token,1);
             break;
           }
         break;
@@ -923,7 +962,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
       {
         if (LocaleCompare((char *) keyword,"pattern") == 0)
           {
-            policy_info->pattern=ConstantString(token);
+            policy_info->pattern=AcquirePolicyString(token,1);
             break;
           }
         break;
@@ -954,7 +993,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
       {
         if (LocaleCompare((char *) keyword,"value") == 0)
           {
-            policy_info->value=ConstantString(token);
+            policy_info->value=AcquirePolicyString(token,1);
             break;
           }
         break;
@@ -1013,7 +1052,7 @@ MagickPrivate MagickBooleanType PolicyComponentGenesis(void)
 
 static void *DestroyPolicyElement(void *policy_info)
 {
-  register PolicyInfo
+  PolicyInfo
     *p;
 
   p=(PolicyInfo *) policy_info;
@@ -1138,7 +1177,7 @@ static MagickBooleanType SetPolicyValue(const PolicyDomain domain,
   MagickBooleanType
     status;
 
-  register PolicyInfo
+  PolicyInfo
     *p;
 
   status=MagickTrue;
@@ -1163,10 +1202,10 @@ static MagickBooleanType SetPolicyValue(const PolicyDomain domain,
       p->exempt=MagickFalse;
       p->signature=MagickCoreSignature;
       p->domain=domain;
-      p->name=ConstantString(name);
+      p->name=AcquirePolicyString(name,1);
       status=AppendValueToLinkedList(policy_cache,p);
     }
-  p->value=ConstantString(value);
+  p->value=AcquirePolicyString(value,1);
   UnlockSemaphoreInfo(policy_semaphore);
   if (status == MagickFalse)
     p=(PolicyInfo *) RelinquishMagickMemory(p);
@@ -1222,15 +1261,21 @@ MagickExport MagickBooleanType SetMagickSecurityPolicyValue(
     }
     case SystemPolicyDomain:
     {
+      if (LocaleCompare(name,"font") == 0)
+        return(SetPolicyValue(domain,name,value));
       if (LocaleCompare(name,"max-memory-request") == 0)
         {
           current_value=GetPolicyValue("system:max-memory-request");
           if ((current_value == (char *) NULL) ||
               (StringToSizeType(value,100.0) < StringToSizeType(current_value,100.0)))
             {
+              if (current_value != (char *) NULL)
+                current_value=DestroyString(current_value);
               ResetMaxMemoryRequest();
               return(SetPolicyValue(domain,name,value));
             }
+          if (current_value != (char *) NULL)
+            current_value=DestroyString(current_value);
         }
       if (LocaleCompare(name,"memory-map") == 0)
         {
@@ -1249,7 +1294,13 @@ MagickExport MagickBooleanType SetMagickSecurityPolicyValue(
           current_value=GetPolicyValue("system:shred");
           if ((current_value == (char *) NULL) ||
               (StringToInteger(value) > StringToInteger(current_value)))
-            return(SetPolicyValue(domain,name,value));
+            {
+              if (current_value != (char *) NULL)
+                current_value=DestroyString(current_value);
+              return(SetPolicyValue(domain,name,value));
+            }
+          if (current_value != (char *) NULL)
+            current_value=DestroyString(current_value);
         }
       break;
     }
