@@ -83,7 +83,7 @@
 /*
   Macros
 */
-#define SIXEL_RGB(r, g, b) ((int) (((ssize_t) (r) << 16) + ((g) << 8) +  (b)))
+#define SIXEL_RGB(r, g, b) ((int) (((ssize_t) ((r) & 0xff) << 16) + (((g) & 0xff) << 8) +  ((b) & 0xff)))
 #define SIXEL_PALVAL(n,a,m) ((int) (((ssize_t) (n) * (a) + ((m) / 2)) / (m)))
 #define SIXEL_XRGB(r,g,b) SIXEL_RGB(SIXEL_PALVAL(r, 255, 100), SIXEL_PALVAL(g, 255, 100), SIXEL_PALVAL(b, 255, 100))
 
@@ -188,11 +188,11 @@ static int hls_to_rgb(int hue, int lum, int sat)
         magic2=(int) (((ssize_t) lum*((ssize_t) HLSMAX+sat)+(HLSMAX/2))/HLSMAX);
       else
         magic2=(int) (lum+sat-(((ssize_t) lum*sat)+(HLSMAX/2))/HLSMAX);
-      magic1=2*lum-magic2;
-      r=(hue_to_rgb(magic1,magic2,(ssize_t) hue+(HLSMAX/3))*RGBMAX+(HLSMAX/2))/
+      magic1=(int) (2*(ssize_t) lum-magic2);
+      b=(hue_to_rgb(magic1,magic2,(ssize_t) hue+(HLSMAX/3))*RGBMAX+(HLSMAX/2))/
         HLSMAX;
-      g=(hue_to_rgb(magic1,magic2,hue)*RGBMAX+(ssize_t) (HLSMAX/2))/HLSMAX;
-      b=(hue_to_rgb(magic1,magic2,(ssize_t) hue-(HLSMAX/3))*RGBMAX+(HLSMAX/2))/
+      r=(hue_to_rgb(magic1,magic2,hue)*RGBMAX+(ssize_t) (HLSMAX/2))/HLSMAX;
+      g=(hue_to_rgb(magic1,magic2,(ssize_t) hue-(HLSMAX/3))*RGBMAX+(HLSMAX/2))/
         HLSMAX;
     }
   return(SIXEL_RGB(r,g,b));
@@ -1023,9 +1023,11 @@ static MagickBooleanType IsSIXEL(const unsigned char *magick,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exception)
+static Image *ReadSIXELImage(const ImageInfo *image_info,
+  ExceptionInfo *exception)
 {
   char
+    *p,
     *sixel_buffer;
 
   Image
@@ -1034,26 +1036,20 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   MagickBooleanType
     status;
 
-  char
-    *p;
-
-  ssize_t
-    x;
-
   Quantum
     *q;
 
   size_t
     length;
 
+  sixel_pixel_t
+    *sixel_pixels;
+
   ssize_t
     i,
     j,
     y;
 
-  sixel_pixel_t
-    *sixel_pixels;
-
   unsigned char
     *sixel_palette;
 
@@ -1062,11 +1058,11 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1084,30 +1080,35 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   if (sixel_buffer != (char *) NULL)
     while (ReadBlobString(image,p) != (char *) NULL)
     {
+      ssize_t
+       offset;
+
       if ((*p == '#') && ((p == sixel_buffer) || (*(p-1) == '\n')))
         continue;
       if ((*p == '}') && (*(p+1) == ';'))
         break;
       p+=strlen(p);
-      if ((size_t) (p-sixel_buffer+MagickPathExtent+1) < length)
+      offset=p-sixel_buffer;
+      if ((size_t) (offset+MagickPathExtent+1) < length)
         continue;
       length<<=1;
       sixel_buffer=(char *) ResizeQuantumMemory(sixel_buffer,length+
         MagickPathExtent+1,sizeof(*sixel_buffer));
       if (sixel_buffer == (char *) NULL)
         break;
-      p=sixel_buffer+strlen(sixel_buffer);
+      p=sixel_buffer+offset;
     }
   if (sixel_buffer == (char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   sixel_buffer[length]='\0';
   /*
-    Decode SIXEL
+    Decode SIXEL.
   */
   sixel_pixels=(sixel_pixel_t *) NULL;
-  if (sixel_decode(image,(unsigned char *) sixel_buffer,&sixel_pixels,
+  status=sixel_decode(image,(unsigned char *) sixel_buffer,&sixel_pixels,
     &image->columns,&image->rows,&sixel_palette,&image->colors,
-    exception) == MagickFalse)
+    exception);
+  if (status == MagickFalse)
     {
       sixel_buffer=(char *) RelinquishMagickMemory(sixel_buffer);
       if (sixel_pixels != (sixel_pixel_t *) NULL)
@@ -1130,10 +1131,11 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
       sixel_palette=(unsigned char *) RelinquishMagickMemory(sixel_palette);
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  for (i = 0; i < (ssize_t) image->colors; ++i) {
-    image->colormap[i].red   = ScaleCharToQuantum(sixel_palette[i * 4 + 0]);
-    image->colormap[i].green = ScaleCharToQuantum(sixel_palette[i * 4 + 1]);
-    image->colormap[i].blue  = ScaleCharToQuantum(sixel_palette[i * 4 + 2]);
+  for (i = 0; i < (ssize_t) image->colors; ++i)
+  {
+    image->colormap[i].red=ScaleCharToQuantum(sixel_palette[i * 4 + 0]);
+    image->colormap[i].green=ScaleCharToQuantum(sixel_palette[i * 4 + 1]);
+    image->colormap[i].blue=ScaleCharToQuantum(sixel_palette[i * 4 + 2]);
   }
   j=0;
   if (image_info->ping == MagickFalse)
@@ -1143,12 +1145,15 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
       */
       for (y=0; y < (ssize_t) image->rows; y++)
       {
+        ssize_t
+          x;
+
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (Quantum *) NULL)
           break;
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          j=(ssize_t) sixel_pixels[y * image->columns + x];
+          j=(ssize_t) sixel_pixels[y*image->columns+x];
           j=ConstrainColormapIndex(image,j,exception);
           SetPixelIndex(image,j,q);
           SetPixelRed(image,image->colormap[j].red,q);
@@ -1303,7 +1308,7 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
